
local require = GLOBAL.require
local STRINGS = GLOBAL.STRINGS
local RECIPETABS = GLOBAL.RECIPETABS
local Ingredient = GLOBAL.Ingredient
local FOODTYPE = GLOBAL.FOODTYPE
local TECH = GLOBAL.TECH
local TUNING = GLOBAL.TUNING
local TheSim = GLOBAL.TheSim
local Vector3 = GLOBAL.Vector3
local ACTIONS = GLOBAL.ACTIONS
local TheNet = GLOBAL.TheNet

PrefabFiles =
{
    "largechest",
    "largeicebox"
}

Assets =
{
    Asset("ATLAS", "images/inventoryimages/largechest.xml"),
    Asset("IMAGE", "images/inventoryimages/largechest.tex"),
    Asset("ATLAS", "images/inventoryimages/largeicebox.xml"),
    Asset("IMAGE", "images/inventoryimages/largeicebox.tex"),
    Asset("ANIM", "anim/ui_largechest_5x5.zip")
}

--------------------------------------------------------------------------

-- Source modified from containers.lua

local containers = require "containers"

local params = {}

local containers_widgetsetup_base = containers.widgetsetup
function containers.widgetsetup(container, prefab, data, ...)
    local t = params[prefab or container.inst.prefab]
    if t ~= nil then
        for k, v in pairs(t) do
            container[k] = v
        end
        container:SetNumSlots(container.widget.slotpos ~= nil and #container.widget.slotpos or 0)
    else
        containers_widgetsetup_base(container, prefab, data, ...)
    end
end

local function makeChest()
    local container =
    {
        widget =
        {
            slotpos = {},
            animbank = "ui_largechest_5x5",
            animbuild = "ui_largechest_5x5",
            pos = GLOBAL.Vector3(0, 200, 0),
            side_align_tip = 160,
        },
        type = "chest",
    }

    for y = 3, -1, -1 do
        for x = -1, 3 do
            table.insert(container.widget.slotpos, GLOBAL.Vector3(80 * x - 80 * 2 + 80, 80 * y - 80 * 2 + 80, 0))
        end
    end

    return container
end

params.largechest = makeChest()
params.largeicebox = makeChest()

function params.largeicebox.itemtestfn(container, item, slot)
    if item:HasTag("icebox_valid") then
        return true
    end

    --Perishable
    if not (item:HasTag("fresh") or item:HasTag("stale") or item:HasTag("spoiled")) then
        return false
    end

    --Edible
    for k, v in pairs(FOODTYPE) do
        if item:HasTag("edible_"..v) then
            return true
        end
    end

    return false
end

for k, v in pairs(params) do
    containers.MAXITEMSLOTS = math.max(containers.MAXITEMSLOTS, v.widget.slotpos ~= nil and #v.widget.slotpos or 0)
end

--------------------------------------------------------------------------


local icebox_ingredients = { Ingredient("goldnugget", 6), Ingredient("gears", 3), Ingredient("cutstone", 3) }

local chest_ingredients = { Ingredient("boards", 9) }

local function largechest_recipe(ingredients, level)
    AddRecipe("largechest", ingredients, RECIPETABS.TOWN, level, "largechest_placer",
        2, nil, nil, nil, "images/inventoryimages/largechest.xml")
end

local function largeicebox_recipe(ingredients, level)
     AddRecipe("largeicebox", ingredients, RECIPETABS.FARM, level, "largeicebox_placer",
        2, nil, nil, nil, "images/inventoryimages/largeicebox.xml")
end

largeicebox_recipe(icebox_ingredients, TECH.SCIENCE_TWO)
largechest_recipe(chest_ingredients, TECH.SCIENCE_TWO)

STRINGS.NAMES.LARGECHEST = "Large Chest"
STRINGS.RECIPE_DESC.LARGECHEST = "A large chest to put more stuff"
STRINGS.CHARACTERS.GENERIC.DESCRIBE.LARGECHEST = "Looks so fancy!"

STRINGS.NAMES.LARGEICEBOX = "Large Icebox"
STRINGS.RECIPE_DESC.LARGEICEBOX = "A gigantic icebox."
STRINGS.CHARACTERS.GENERIC.DESCRIBE.LARGEICEBOX = "I have harnessed the power of cold!"
